/*
 * $Id: tarpon100.c,v 1.2 2004/05/19 18:03:42 linville Exp $
 *
 * Normal mappings of chips in physical memory
 *
 * Modified for Runtop Tarpon-100 partitions... -- JWL
 */

/* partition support */
#define HAVE_PARTITIONS

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <asm/io.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/config.h>
#ifdef HAVE_PARTITIONS
#include <linux/mtd/partitions.h>
#endif

#define WINDOW_ADDR	0x1c000000
#define WINDOW_SIZE	0x00800000
#define BUSWIDTH	2

#define U_BOOT_START		0x00000000
#define U_BOOT_LEN		0x00030000

#define U_BOOT_ENV_START	0x00030000
#define U_BOOT_ENV_LEN		0x00010000

#define APP_START		0x00040000
#define APP_LEN			0x005c0000

#define SYSTEM_START		0x00600000
#define SYSTEM_LEN		0x00200000

#ifdef HAVE_PARTITIONS
static struct mtd_partition tarpon100_partitions[] =
{
	/* u-boot */
	{
		      name: "U-Boot",
		    offset: U_BOOT_START,
		      size: U_BOOT_LEN,
		mask_flags: 0
	},
	/* u-boot environment */
	{
		      name: "U-Boot Environment",
		    offset: U_BOOT_ENV_START,
		      size: U_BOOT_ENV_LEN,
		mask_flags: 0
	},
	/* app filesystem */
	{
		      name: "Application Filesystem",
		    offset: APP_START,
		      size: APP_LEN,
		mask_flags: 0
	},
	/* system (kernel + ramdisk) */
	{
		      name: "System (Kernel + Ramdisk)",
		    offset: SYSTEM_START,
		      size: SYSTEM_LEN,
		mask_flags: 0
	}
};

#define NUM_PARTITIONS	(sizeof(tarpon100_partitions) \
                       / sizeof(struct mtd_partition))
#endif

static struct mtd_info *mymtd;

__u8 tarpon100_read8(struct map_info *map, unsigned long ofs)
{
	return __raw_readb(map->map_priv_1 + ofs);
}

__u16 tarpon100_read16(struct map_info *map, unsigned long ofs)
{
	return __raw_readw(map->map_priv_1 + ofs);
}

__u32 tarpon100_read32(struct map_info *map, unsigned long ofs)
{
	return __raw_readl(map->map_priv_1 + ofs);
}

void tarpon100_copy_from(struct map_info *map, void *to, unsigned long from, ssize_t len)
{
	memcpy_fromio(to, map->map_priv_1 + from, len);
}

void tarpon100_write8(struct map_info *map, __u8 d, unsigned long adr)
{
	__raw_writeb(d, map->map_priv_1 + adr);
	mb();
}

void tarpon100_write16(struct map_info *map, __u16 d, unsigned long adr)
{
	__raw_writew(d, map->map_priv_1 + adr);
	mb();
}

void tarpon100_write32(struct map_info *map, __u32 d, unsigned long adr)
{
	__raw_writel(d, map->map_priv_1 + adr);
	mb();
}

void tarpon100_copy_to(struct map_info *map, unsigned long to, const void *from, ssize_t len)
{
	memcpy_toio(map->map_priv_1 + to, from, len);
}

struct map_info tarpon100_map = {
	name: "Tarpon-100 Flash",
	size: WINDOW_SIZE,
	buswidth: BUSWIDTH,
	read8: tarpon100_read8,
	read16: tarpon100_read16,
	read32: tarpon100_read32,
	copy_from: tarpon100_copy_from,
	write8: tarpon100_write8,
	write16: tarpon100_write16,
	write32: tarpon100_write32,
	copy_to: tarpon100_copy_to
};

int __init init_tarpon100(void)
{
       	printk(KERN_NOTICE "tarpon100 flash device: %x at %x\n", WINDOW_SIZE, WINDOW_ADDR);
	tarpon100_map.map_priv_1 = (unsigned long)ioremap(WINDOW_ADDR, WINDOW_SIZE);

	if (!tarpon100_map.map_priv_1) {
		printk("Failed to ioremap\n");
		return -EIO;
	}
	mymtd = do_map_probe("cfi_probe", &tarpon100_map);
	if (mymtd) {
		mymtd->module = THIS_MODULE;

#if defined(HAVE_PARTITIONS)
		add_mtd_partitions(mymtd, tarpon100_partitions,
		                   NUM_PARTITIONS);
#else
		add_mtd_device(mymtd);
#endif
		return 0;
	}

	iounmap((void *)tarpon100_map.map_priv_1);
	return -ENXIO;
}

static void __exit cleanup_tarpon100(void)
{
	if (mymtd) {
#if defined(HAVE_PARTITIONS)
		del_mtd_device(mymtd);
#else
		del_mtd_partitions(mymtd);
#endif
		map_destroy(mymtd);
	}
	if (tarpon100_map.map_priv_1) {
		iounmap((void *)tarpon100_map.map_priv_1);
		tarpon100_map.map_priv_1 = 0;
	}
}

module_init(init_tarpon100);
module_exit(cleanup_tarpon100);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("John W. Linville <linville@lvl7.com>");
MODULE_DESCRIPTION("MTD map driver for Runtop Tarpon-100");
