/*
 * Early initialization code for BCM94710 boards
 *
 * Copyright (C) 2004 Broadcom Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <asm/io.h>
#include <asm/bootinfo.h>

extern char arcs_cmdline[CL_SIZE];

#ifdef CONFIG_BLK_DEV_INITRD
extern unsigned long initrd_start, initrd_end;
#endif

void __init prom_init(int argc, char **argv, char **envp, int *prom_vec)
{
	unsigned long mem, before, offset;
    unsigned long val;
    int i;
    char *argp;

	mips_machgroup = MACH_GROUP_BRCM;
	mips_machtype  = MACH_BCM947XX;

	/* Figure out memory size by finding aliases.
	 *
	 * We assume that there will be no more than 128 MB of memory,
	 * and that the memory size will be a multiple of 1 MB.
	 *
	 * We set 'before' to be the amount of memory (in MB) before this
	 * function, i.e. one MB less than the number  of MB of memory that we
	 * *know* we have.  And we set 'offset' to be the address of 'prominit'
	 * minus 'before', so that KSEG0 or KSEG1 base + offset < 1 MB.
	 * This prevents us from overrunning 128 MB and causing a bus error.
	 */
	before = ((unsigned long) &prom_init) & (127 << 20);
	offset = ((unsigned long) &prom_init) - before;
	for (mem = before + (1 << 20); mem < (128 << 20); mem += (1 << 20)) {
		if (*(unsigned long *)(offset + mem) ==
		    *(unsigned long *)(prom_init)) {
			/*
			 * We may already be well past the end of memory at
			 * this point, so we'll have to compensate for it.
			 */
			mem -= before;
			break;
		}
    }
	
    add_memory_region(0, mem, BOOT_MEM_RAM);
    printk ("Setting memory region to 0x%lx\n", mem);

    argp = arcs_cmdline + strlen(arcs_cmdline);

    for (i = 0; i < argc; i++) {
        if (argv[i] != NULL) {
            *argp++ = ' ';
            *argp = '\0';
            strcat(argp, argv[i]);
            argp += strlen(argv[i]);
        }
    }

    if (envp[i = 0] == NULL) {
        printk ("\t envp[0] is NULL\n");
        return;
    }

    do {
        printk ("Environment [%d] : <%s>\n", i, envp[i]);
        if ((argp = strstr (envp[i], "baudrate=")) != NULL) {
            sscanf (argp, "baudrate=%ld", &val);
            if ((argp = strstr(arcs_cmdline, "console=")) == NULL) {
                sprintf(arcs_cmdline, "%s console=ttyS0,%ld", arcs_cmdline, val);
            }
            continue;
        }

#ifdef CONFIG_BLK_DEV_INITRD
        if ((argp = strstr (envp[i], "initrd_start=")) != NULL) {
            sscanf (argp, "initrd_start=0x%x", (int *)&val);
            initrd_start = KSEG0ADDR (val);
            continue;
        }

        if ((argp = strstr (envp[i], "initrd_size=")) != NULL) {
            /* conversion doesn't work if first
               letter is not a decimal digit... (?) */
            *(argp + strlen ("initrd_size=0")) = '0';
            sscanf (argp, "initrd_size=%x", (int *)&val);
            initrd_end = KSEG0ADDR (initrd_start + val);
            continue;
        }
#endif
    } while (envp[++i] != NULL);
}

void __init prom_free_prom_memory(void)
{
}
