/*
 * arch/ppc/platforms/argan.h
 *
 * Definitions for IBM Argan 
 *
 * Author: <source@mvista.com>
 *
 * Based on Artesyn Katana code done by Tim Montgomery <timm@artesyncp.com>
 * Based on code done by Rabeeh Khoury - rabeeh@galileo.co.il 
 * Based on code done by Mark A. Greer <mgreer@mvista.com>
 *
 * <2004> (c) MontaVista Software, Inc. This file is licensed under
 * the terms of the GNU General Public License version 2. This program
 * is licensed "as is" without any warranty of any kind, whether express
 * or implied.
 */

/*
 * This is the CPU physical memory map (windows must be at least 1MB and start
 * on a boundary that is a multiple of the window size):
 *
 * Seems on the Argan the MV64360 Registers can be in only 2 places
 * per switch U17 0x14000000 or 0xf1000000 easily - chose to implement
 * at 0xf1000000 only at this time
 *
 *    0xfff00000-0xffffffff      - 8 Flash
 *    0xffe00000-0xffefffff      - SRAM
 *    0xffd00000-0xffd00004      - CPLD
 *    0xffc00000-0xffc0000f      - UART
 *    0xffb00000-0xffb07fff      - FRAM  
 *    0xffa00000-0xffafffff      - *** HOLE ***
 *    0xff900000-0xff9fffff      - MV64360 Integrated SRAM
 *    0xfe000000-0xff8fffff      - *** HOLE ***
 *    0xfc000000-0xfdffffff      - 32bit Flash
 *    0xf1010000-0xfbffffff      - *** HOLE ***
 *    0xf1000000-0xf100ffff      - MV64360 Registers
 */

#ifndef __PPC_PLATFORMS_ARGAN_H__
#define __PPC_PLATFORMS_ARGAN_H__

#define ARGAN_BOOT_8BIT_BASE		0xfff00000
#define ARGAN_BOOT_8BIT_SIZE 		_1M
#define ARGAN_BOOT_SRAM_BASE 		0xffe00000
#define ARGAN_BOOT_SRAM_SIZE 		_1M
#define ARGAN_CPLD_BASE 		0xffd00000
#define ARGAN_CPLD_REG3 		0xffd00003
#define ARGAN_CPLD_SIZE 		5
#define ARGAN_UART_BASE 		0xffc00000
#define ARGAN_FRAM_BASE 		0xffb00000
#define ARGAN_BRIDGE_REG_BASE 		0xf1000000
#define ARGAN_BRIDGE_REG_SIZE 		_64K
#define ARGAN_INTERNAL_SRAM_BASE 	0xff900000
#define ARGAN_INTERNAL_SRAM_SIZE 	_256K
#define ARGAN_32BIT_BASE 		0xfc000000
#define ARGAN_32BIT_SIZE 		_32M

#define ARGAN_BUS_SPEED 		133333333
#define ARGAN_PIBS_DATABASE 		0xf0000 /* from PIBS src code */

#define MV64360_ETH_PORT_SERIAL_CONTROL_REG_PORT0 	0x243c 
#define MV64360_ETH_PORT_SERIAL_CONTROL_REG_PORT1 	0x283c 

#endif /* __PPC_PLATFORMS_ARGAN_H__ */
