/*
    i2c-mpc824x.c - Adapter driver for I2C support on Motorola MPC824x

    Copyright (c) 2002 NetBotz Inc <primm@netbotz.com>

    Derived from i2c-via.c

    Copyright (c) 1998, 1999 Kysti M�kki <kmalkki@cc.hut.fi>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/module.h>
#include <linux/pci.h>
#include <asm/io.h>
#include <linux/types.h>
#include <linux/i2c.h>
#include <linux/i2c-algo-mpc824x.h>
#include <linux/init.h>

#define DRV_VERSION	"0.1"

#ifdef MODULE_LICENSE
MODULE_LICENSE("GPL");
#endif

/* ----- local functions ----------------------------------------------	*/
static void i2c_mpc824x_inc(struct i2c_adapter *adapter)
{
	MOD_INC_USE_COUNT;
}

static void i2c_mpc824x_dec(struct i2c_adapter *adapter)
{
	MOD_DEC_USE_COUNT;
}

/* ------------------------------------------------------------------------ */

static struct i2c_algo_mpc824x_data i2c_mpc824x_data = { 0 };

static struct i2c_adapter i2c_mpc824x_ops = {
	"Motorola mpc824X i2c",
	I2C_HW_MPC824X,
	NULL,
	&i2c_mpc824x_data,
	i2c_mpc824x_inc,
	i2c_mpc824x_dec,
	NULL,
	NULL,
};

#ifdef MODULE
static
#else
extern
#endif
int __init i2c_mpc824x_init(void)
{
	printk("i2c-mpc824x.o version %s\n", DRV_VERSION);
	if (i2c_mpc824x_add_bus(&i2c_mpc824x_ops) == 0) {
		printk("i2c-mpc824x.o: Module succesfully loaded\n");
		return 0;
	} else {
		printk("i2c-mpc824x.o: Algo-mpc824x error, couldn't register bus\n");
		return -ENODEV;
	}
}

EXPORT_NO_SYMBOLS;

#ifdef MODULE
MODULE_AUTHOR("Michael Primm <primm@netbotz.com>");
MODULE_DESCRIPTION("i2c for Motorola mpc824x");

int init_module(void)
{
	return i2c_mpc824x_init();
}

void cleanup_module(void)
{
	i2c_mpc824x_del_bus(&i2c_mpc824x_ops);
}
#endif
