/*
 * include/asm-arm/arch-ixp2000/serial.h
 *
 * Serial port defn for ixp2000 based systems
 *
 * Author: Deepak Saxena <dsaxena@mvsita.com>
 *
 * Copyright (c) 2002 MontaVista Software, Inc.
 *
 * Changelog:
 *  28-Mar-2003: Andrzej Mialkowski <andrzej.mialkowski@intel.com>
 *     - Added support for IXDP2401 additional serial ports
 *     - Support for little-endian mode (not tested)
 *     - Added board parameters dependency, cleaned
 */

#define BASE_BAUD (CONFIG_IXP2000_EXT_XTAL / (2 * 16))

#define STD_COM_FLAGS	(ASYNC_SKIP_TEST)

#define	IXP2000_STD_SERIAL_PORT_DEFNS		\
	{					\
		type: PORT_XSCALE,		\
		xmit_fifo_size: 16,		\
		baud_base: BASE_BAUD,		\
		irq: IRQ_IXP2000_UART,		\
		flags: STD_COM_FLAGS,		\
		io_type: SERIAL_IO_MEM32,	\
                iomem_base: (u32*)UART_VIRT_BASE,\
		iomem_reg_shift: 2		\
	}

#if defined (CONFIG_ARCH_IXDP2401)

#define BASE_BAUD_ADD ( 1843200 / 16 )
#define RS_TABLE_SIZE   3

#define	IXP2000_ADD_SERIAL_PORT_DEFNS		\
   {					        \
                type: PORT_16550A,              \
                xmit_fifo_size: 16,             \
                baud_base: BASE_BAUD_ADD,       \
                irq: IRQ_IXDP2401_ADD_UART,     \
	        flags: STD_COM_FLAGS,		\
                iomem_base: (u32*)IXDP2401_VIRT_ADD_UART_BASE, \
		io_type: SERIAL_IO_MEM32,	\
		iomem_reg_shift: 2		\
    }, \
    {					        \
                type: PORT_16550A,              \
                xmit_fifo_size: 16,             \
                baud_base: BASE_BAUD_ADD,       \
                irq: IRQ_IXDP2401_SAUS_UART,    \
		flags: STD_COM_FLAGS,		\
                iomem_base: (u32*)IXDP2401_VIRT_SAUS_UART_BASE, \
		io_type: SERIAL_IO_MEM32,	\
		iomem_reg_shift: 2		\
    }
#elif defined (CONFIG_ARCH_IXDP2801)

#define BASE_BAUD_ADD ( 1843200 / 16 )
#define RS_TABLE_SIZE   3

#define	IXP2000_ADD_SERIAL_PORT_DEFNS		\
   {					        \
                type: PORT_16550A,              \
                xmit_fifo_size: 16,             \
                baud_base: BASE_BAUD_ADD,       \
                irq: IRQ_IXDP2801_ADD_UART,     \
	        flags: STD_COM_FLAGS,		\
                iomem_base: (u32*)IXDP2801_VIRT_ADD_UART_BASE, \
		io_type: SERIAL_IO_MEM32,	\
		iomem_reg_shift: 2		\
    }, \
    {					        \
                type: PORT_16550A,              \
                xmit_fifo_size: 16,             \
                baud_base: BASE_BAUD_ADD,       \
                irq: IRQ_IXDP2801_SAUS_UART,    \
		flags: STD_COM_FLAGS,		\
                iomem_base: (u32*)IXDP2801_VIRT_SAUS_UART_BASE, \
		io_type: SERIAL_IO_MEM32,	\
		iomem_reg_shift: 2		\
    }
#else
#define RS_TABLE_SIZE   1
#define	IXP2000_ADD_SERIAL_PORT_DEFNS
#endif

#define STD_SERIAL_PORT_DEFNS IXP2000_STD_SERIAL_PORT_DEFNS, IXP2000_ADD_SERIAL_PORT_DEFNS
#define EXTRA_SERIAL_PORT_DEFNS

